﻿' File Class1.cs
' A C# class library for a hypothetical two-card poker game
' Used as a target for IronPython test automation in harness.py


Imports Microsoft.VisualBasic
Imports System
'using System.Collections.Generic;
'using System.Linq;
'using System.Text;

Namespace TwoCardPokerLib
  Public Class Card
	Private rank_Renamed As String
	Private suit_Renamed As String
	Public Sub New()
	  Me.rank_Renamed = "A" ' A, 2, 3,.. ,9, T, J, Q, K
	  Me.suit_Renamed = "s" ' c, d, h, s
	End Sub
	Public Sub New(ByVal s As String)
	  Me.rank_Renamed = s.Chars(0).ToString()
	  Me.suit_Renamed = s.Chars(1).ToString()
	End Sub
	Public Overrides Function ToString() As String
	  Return Me.rank_Renamed & Me.suit_Renamed
	End Function
	Public ReadOnly Property Rank() As String
	  Get
		  Return Me.rank_Renamed
	  End Get
	End Property
	Public ReadOnly Property Suit() As String
	  Get
		  Return Me.suit_Renamed
	  End Get
	End Property
	Public Shared Function Beats(ByVal c1 As Card, ByVal c2 As Card) As Boolean
	  If c1.rank = "A" Then
		If c2.rank <> "A" Then
		  Return True
		Else
		  Return False
		End If
	  ElseIf c1.rank = "K" Then
		If c2.rank = "A" OrElse c2.rank = "K" Then
		  Return False
		Else
		  Return True
		End If
	  ElseIf c1.rank = "Q" Then
		If c2.rank = "A" OrElse c2.rank = "K" OrElse c2.rank = "Q" Then
		  Return False
		Else
		  Return True
		End If
	  ElseIf c1.rank = "J" Then
		If c2.rank = "A" OrElse c2.rank = "K" OrElse c2.rank = "Q" OrElse c2.rank = "J" Then
		  Return False
		Else
		  Return True
		End If
	  ElseIf c1.rank = "T" Then
		If c2.rank = "A" OrElse c2.rank = "K" OrElse c2.rank = "Q" OrElse c2.rank = "J" OrElse c2.rank = "T" Then
		  Return False
		Else
		  Return True
		End If
	  Else
		Dim c1Rank As Integer = Integer.Parse(c1.rank)
		Dim c2Rank As Integer = Integer.Parse(c2.rank)
		Return c1Rank > c2Rank
	  End If
	End Function ' Beats()

	Public Shared Function Ties(ByVal c1 As Card, ByVal c2 As Card) As Boolean
	  Return (c1.rank = c2.rank)
	End Function
  End Class ' class Card

  Public Class Hand
	Private card1 As Card ' high card
	Private card2 As Card ' low card
	Public Sub New()
	  Me.card1 = New Card() ' high card
	  Me.card2 = New Card() ' low card
	End Sub
	Public Sub New(ByVal c1 As Card, ByVal c2 As Card)
	  Me.card1 = New Card(c1.Rank + c1.Suit)
	  Me.card2 = New Card(c2.Rank + c2.Suit)
	End Sub
	Public Sub New(ByVal s1 As String, ByVal s2 As String)
	  Me.card1 = New Card(s1)
	  Me.card2 = New Card(s2)
	End Sub
	Public Overrides Function ToString() As String
	  Return Me.card1.ToString() & "-" & Me.card2.ToString()
	End Function

	Private Function IsPair() As Boolean
		Return Me.card1.Rank = Me.card2.Rank
	End Function
	Private Function IsFlush() As Boolean
		Return Me.card1.Suit = Me.card2.Suit
	End Function
	Private Function IsStraight() As Boolean
	  If Me.card1.Rank = "A" AndAlso Me.card2.Rank = "K" Then ' boundary
		  Return True
	  ElseIf Me.card1.Rank = "K" AndAlso Me.card2.Rank = "Q" Then
		  Return True
	  ElseIf Me.card1.Rank = "Q" AndAlso Me.card2.Rank = "J" Then
		  Return True
	  ElseIf Me.card1.Rank = "J" AndAlso Me.card2.Rank = "T" Then
		  Return True
	  ElseIf Me.card1.Rank = "T" AndAlso Me.card2.Rank = "9" Then
		  Return True
	  ElseIf Me.card1.Rank = "9" AndAlso Me.card2.Rank = "8" Then
		  Return True
	  ElseIf Me.card1.Rank = "8" AndAlso Me.card2.Rank = "7" Then
		  Return True
	  ElseIf Me.card1.Rank = "7" AndAlso Me.card2.Rank = "6" Then
		  Return True
	  ElseIf Me.card1.Rank = "6" AndAlso Me.card2.Rank = "5" Then
		  Return True
	  ElseIf Me.card1.Rank = "5" AndAlso Me.card2.Rank = "4" Then
		  Return True
	  ElseIf Me.card1.Rank = "4" AndAlso Me.card2.Rank = "3" Then
		  Return True
	  ElseIf Me.card1.Rank = "3" AndAlso Me.card2.Rank = "2" Then
		  Return True
	  ElseIf Me.card1.Rank = "A" AndAlso Me.card2.Rank = "2" Then ' boundary
		  Return True
	  Else
		  Return False
	  End If
	End Function
	Private Function IsStraightFlush() As Boolean
		Return Me.IsStraight() AndAlso Me.IsFlush()
	End Function

	Private Function Beats(ByVal h As Hand) As Boolean ' is this > h??
	  ' Straight Flush > Pair > Flush > Straight > Ace high > King high > . . > Four high
	  If Me.IsStraightFlush() Then
		If (Not h.IsStraightFlush()) Then ' this is a straight flush and h isn't
			Return True
		End If
		If h.IsStraightFlush() Then ' both hands are straight flushes..
		  If Card.Beats(Me.card1, h.card1) Then ' this is a higher straight flush
			Return True
		  Else
			Return False
		  End If
		End If ' this.IsStraightFlush()
	  ElseIf Me.IsPair() Then
		If h.IsStraightFlush() Then ' h has a straight flush
			Return False
		End If
		If (Not h.IsPair()) Then
			Return True
		End If
		If h.IsPair() Then ' both hands are pairs
		  If Card.Beats(Me.card1, h.card1) Then ' this is a higher pair
			Return True
		  Else
			Return False ' this is a lower or equal pair
		  End If
		End If ' this.IsPair
	  ElseIf Me.IsFlush() Then
		If h.IsStraightFlush() Then ' h has a straight flush
			Return False
		End If
		If h.IsPair() Then ' h has a pair
			Return False
		End If
		If (Not h.IsFlush()) Then
			Return True
		End If
		If h.IsFlush() Then ' both hands are flushes
		  If Card.Beats(Me.card1, h.card1) Then ' this is a higher flush on first card
			Return True
		  ElseIf Card.Ties(Me.card1, Me.card1) AndAlso Card.Beats(Me.card2, h.card2) Then ' this is a higher flush on 2nd card
			Return True
		  Else
			Return False ' this is a lower or equal flush
		  End If
		End If ' this.IsFlush
	  ElseIf Me.IsStraight() Then
		If h.IsStraightFlush() Then ' h has a straight flush
			Return False
		End If
		If h.IsPair() Then ' h has a pair
			Return False
		End If
		If h.IsFlush() Then ' h has a flush
			Return False
		End If
		If (Not h.IsStraight()) Then
			Return True
		End If
		If h.IsStraight() Then ' both hands are straights
		  If Card.Beats(Me.card1, h.card1) Then ' this is a higher straight
			Return True
		  Else
			Return False ' this is a lower or equal straight
		  End If
		End If ' this.IsStraight
	  Else ' this is Ace-high down to Four-high ( Three-high and Two-high impossible)
		If Card.Beats(Me.card1, h.card1) Then
		  Return True
		ElseIf Card.Beats(h.card1, Me.card1) Then
		  Return False
		ElseIf Card.Ties(Me.card1, h.card1) Then ' this.card1 == h.card1
		  If Card.Beats(Me.card2, h.card2) Then
			Return True
		  ElseIf Card.Beats(h.card2, Me.card2) Then
			Return False
		  End If
		End If
	  End If ' this is Ace-high down to Four-high
	  Return False
	End Function ' Hand.Beats

	Private Function Ties(ByVal h As Hand) As Boolean ' does this tie h?
	  If Me.IsStraightFlush() AndAlso h.IsStraightFlush() AndAlso Me.card1.Rank = h.card1.Rank Then
		Return True
	  ElseIf Me.IsPair() AndAlso h.IsPair() AndAlso Me.card1.Rank = h.card1.Rank Then
		Return True
	  ElseIf Me.IsFlush() AndAlso h.IsFlush() AndAlso Me.card1.Rank = h.card1.Rank Then ' deliberate error; must check 2nd card too
		Return True
	  'else if (this.IsFlush() && h.IsFlush() && this.card1.Rank == h.card1.Rank && this.card2.Rank == h.card2.Rank)
	  '  return true;
	  ElseIf Me.IsStraight() AndAlso h.IsStraight() AndAlso Me.card1.Rank = h.card1.Rank Then
		Return True
	  ElseIf Me.card1.Rank = h.card1.Rank AndAlso Me.card2.Rank = h.card2.Rank Then
		Return True
	  Else
		Return False
	  End If
	End Function ' Hand.Ties

	Public Function Compare(ByVal h As Hand) As Integer
	  If Me.Beats(h) Then
		Return 1
	  ElseIf Me.Ties(h) Then
		Return 0
	  ElseIf h.Beats(Me) Then
		Return -1
	  Else
		Throw New Exception("Illegal path in Compare()")
	  End If
	End Function

	Public Enum HandType
	  StraightFlush
	  Pair
	  Flush
	  Straight
	  AceHigh
	  KingHigh
	  QueenHigh
	  TenHigh
	  NineHigh
	  EightHigh
	  SevenHigh
	  SixHigh
	  FiveHigh
	  FourHigh
	  Invalid
	End Enum

  End Class ' class Hand
End Namespace ' ns
